<?php

/**
 * Users module
 *
 * @link       https://www.fredericgilles.net/fg-prestashop-to-woocommerce/
 * @since      2.0.0
 *
 * @package    FG_PrestaShop_to_WooCommerce_Premium
 * @subpackage FG_PrestaShop_to_WooCommerce_Premium/admin
 */

if ( !class_exists('FG_PrestaShop_to_WooCommerce_Users', false) ) {

	/**
	 * Users class
	 *
	 * @package    FG_PrestaShop_to_WooCommerce_Premium
	 * @subpackage FG_PrestaShop_to_WooCommerce_Premium/admin
	 * @author     Frédéric GILLES
	 */
	class FG_PrestaShop_to_WooCommerce_Users {

		private $users = array();
		private $users_count = 0;

		/**
		 * Initialize the class and set its properties.
		 *
		 * @since    2.0.0
		 * @param    object    $plugin       Admin plugin
		 */
		public function __construct( $plugin ) {

			$this->plugin = $plugin;

		}

		/**
		 * Update the number of total elements found in PrestaShop
		 * 
		 * @since 3.0.0
		 * 
		 * @param int $count Number of total elements
		 * @return int Number of total elements
		 */
		public function get_total_elements_count($count) {
			if ( !isset($this->plugin->premium_options['skip_users']) || !$this->plugin->premium_options['skip_users'] ) {
				$count += $this->plugin->get_employees_count();
				$count += $this->plugin->get_customers_count();
			}
			return $count;
		}

		/**
		 * Delete all users except the current user
		 *
		 */
		public function delete_users($action) {
			global $wpdb;
			
			$sql_queries = array();

			if ( $action == 'all' ) {
				
				// Delete all users except the current user
				$current_user = get_current_user_id();
				if ( is_multisite() ) {
					$blogusers = get_users(array('exclude' => $current_user));
					foreach ( $blogusers as $user ) {
						wp_delete_user($user->ID);
					}
				} else { // monosite (quicker)
					$sql_queries[] = <<<SQL
-- Delete User meta
DELETE FROM $wpdb->usermeta
WHERE user_id != '$current_user'
SQL;

				$sql_queries[] = <<<SQL
-- Delete Users
DELETE FROM $wpdb->users
WHERE ID != '$current_user'
SQL;

					// Execute SQL queries
					if ( count($sql_queries) > 0 ) {
						foreach ( $sql_queries as $sql ) {
							$wpdb->query($sql);
						}
					}
				}
				$this->reset_users_autoincrement();
				
			} else {
				
				// Delete only the imported users
				
				// Truncate the temporary table
				$sql_queries[] = <<<SQL
TRUNCATE {$wpdb->prefix}fg_data_to_delete;
SQL;
				
				// Insert the imported users IDs in the temporary table
				$sql_queries[] = <<<SQL
INSERT IGNORE INTO {$wpdb->prefix}fg_data_to_delete (`id`)
SELECT user_id FROM $wpdb->usermeta
WHERE meta_key LIKE '_fgp2wc_%'
SQL;
				
				$sql_queries[] = <<<SQL
-- Delete Users and user metas
DELETE u, um
FROM $wpdb->users u
LEFT JOIN $wpdb->usermeta um ON um.user_id = u.ID
INNER JOIN {$wpdb->prefix}fg_data_to_delete del
WHERE u.ID = del.id;
SQL;

				// Execute SQL queries
				if ( count($sql_queries) > 0 ) {
					foreach ( $sql_queries as $sql ) {
						$wpdb->query($sql);
					}
				}

			}
			wp_cache_flush();
			
			// Reset the PrestaShop last imported user ID
			update_option('fgp2wc_last_user_id', 0);

			$this->plugin->display_admin_notice(__('Users deleted', $this->plugin->get_plugin_name()));
		}

		/**
		 * Reset the wp_users autoincrement
		 */
		private function reset_users_autoincrement() {
			global $wpdb;

			$sql = "SELECT IFNULL(MAX(ID), 0) + 1 FROM $wpdb->users";
			$max_id = $wpdb->get_var($sql);
			$sql = "ALTER TABLE $wpdb->users AUTO_INCREMENT = $max_id";
			$wpdb->query($sql);
		}

		/**
		 * Reset the Prestashop last imported customer ID
		 *
		 */
		public function reset_customers() {
			update_option('fgp2wc_last_customer_id', 0);
		}
		
		/**
		 * Import users
		 *
		 */
		public function import_users() {
			
			if ( isset($this->plugin->premium_options['skip_users']) && $this->plugin->premium_options['skip_users'] ) {
				return;
			}
			if ( $this->plugin->import_stopped() ) {
				return;
			}
			do_action('fgp2wcp_pre_import_users');
			
			$this->plugin->log(__('Importing users...', $this->plugin->get_plugin_name()));
			
			$this->import_employees();
			$this->import_customers();
			$this->plugin->display_admin_notice(sprintf(_n('%d user imported', '%d users imported', $this->users_count, $this->plugin->get_plugin_name()), $this->users_count));
		}

		/**
		 * Import employees
		 *
		 */
		private function import_employees() {
			$employees = $this->get_employees();
			$employees_count = count($employees);
			foreach ( $employees as $employee ) {
				$user_id = $this->add_user($employee['firstname'], $employee['lastname'], $employee['email'], $employee['passwd'], $employee['id_employee'], '', 'editor');
				if ( !is_wp_error($user_id) ) {
					// Link between the PrestaShop ID and the WordPress user ID
					add_user_meta($user_id, 'prestashop_employee_id', $employee['id_employee'], true);
				}
			}
			$this->plugin->progressbar->increment_current_count($employees_count);
		}

		/**
		 * Import customers
		 *
		 */
		private function import_customers() {

			// Hook for other actions
			do_action('fgp2wcp_pre_import_customers');
			
			do {
				if ( $this->plugin->import_stopped() ) {
					break;
				}
				$customers = $this->get_customers($this->plugin->chunks_size);
				$customers_count = count($customers);
				foreach ( $customers as $customer ) {
					$user_id = $this->add_user($customer['firstname'], $customer['lastname'], $customer['email'], $customer['passwd'], $customer['id_customer'], '', 'customer');
					if ( !is_wp_error($user_id) ) {
						// Add the customer web site
						wp_update_user(array('ID' => $user_id, 'user_url' => $customer['website']));
						// Link between the PrestaShop ID and the WordPress user ID
						add_user_meta($user_id, 'prestashop_customer_id', $customer['id_customer'], true);
						// Add the address fields
						$address = $this->get_customer_address($customer['id_customer']);
						if ( !empty($address) ) {
							$phone = !empty($address['phone_mobile'])? $address['phone_mobile'] : $address['phone'];
							$state = $this->process_state_code($address['state'], $address['country']);
							update_user_meta($user_id, 'billing_company', $address['company']);
							update_user_meta($user_id, 'billing_last_name', $address['lastname']);
							update_user_meta($user_id, 'billing_first_name', $address['firstname']);
							update_user_meta($user_id, 'billing_phone', $phone);
							update_user_meta($user_id, 'billing_address_1', $address['address1']);
							update_user_meta($user_id, 'billing_address_2', $address['address2']);
							update_user_meta($user_id, 'billing_city', $address['city']);
							update_user_meta($user_id, 'billing_state', $state);
							update_user_meta($user_id, 'billing_country', $address['country']);
							update_user_meta($user_id, 'billing_postcode', $address['postcode']);
							update_user_meta($user_id, 'billing_email', $customer['email']);
							update_user_meta($user_id, 'shipping_company', $address['company']);
							update_user_meta($user_id, 'shipping_last_name', $address['lastname']);
							update_user_meta($user_id, 'shipping_first_name', $address['firstname']);
							update_user_meta($user_id, 'shipping_address_1', $address['address1']);
							update_user_meta($user_id, 'shipping_address_2', $address['address2']);
							update_user_meta($user_id, 'shipping_city', $address['city']);
							update_user_meta($user_id, 'shipping_state', $state);
							update_user_meta($user_id, 'shipping_country', $address['country']);
							update_user_meta($user_id, 'shipping_postcode', $address['postcode']);
						}
						
						do_action('fgp2wcp_post_add_customer', $user_id, $customer);
					}
					// Increment the PrestaShop last imported customer ID
					update_option('fgp2wc_last_customer_id', $customer['id_customer']);
				}
				$this->plugin->progressbar->increment_current_count($customers_count);
			} while ( ($customers != null) && ($customers_count > 0) );
		}

		/**
		 * Add a user if it does not exists
		 *
		 * @param string $firstname User's first name
		 * @param string $lastname User's last name
		 * @param string $email User's email
		 * @param string $password User's password in PrestaShop
		 * @param int $ps_user_id User's id in PrestaShop
		 * @param string $register_date Registration date
		 * @param string $role User's role - default: subscriber
		 * @return int User ID
		 */
		private function add_user($firstname, $lastname, $email='', $password='', $ps_user_id='', $register_date='', $role='subscriber') {
			$email = sanitize_email($email);
			$login = $email; // Use the email as login
			$display_name = $firstname . ' ' . $lastname;

			$user = get_user_by('email', $email);
			if ( !$user ) {
				// Create a new user
				$userdata = array(
					'user_login'		=> $login,
					'user_pass'			=> wp_generate_password( 12, false ),
					'user_nicename'		=> $login,
					'user_email'		=> $email,
					'display_name'		=> $display_name,
					'first_name'		=> $firstname,
					'last_name'			=> $lastname,
					'user_registered'	=> $register_date,
					'role'				=> $role,
				);
				$user_id = wp_insert_user( $userdata );
				if ( is_wp_error($user_id) ) {
//						$this->plugin->display_admin_error(sprintf(__('Creating user %s: %s', $this->plugin->get_plugin_name()), $login, $user_id->get_error_message()));
				} else {
					$this->users_count++;
					add_user_meta($user_id, '_fgp2wc_old_user_id', $ps_user_id, true);
					if ( !empty($password) ) {
						// PrestaShop password to authenticate the users
						add_user_meta($user_id, 'prestashop_pass', $password, true);
					}
//						$this->plugin->display_admin_notice(sprintf(__('User %s created', $this->plugin->get_plugin_name()), $login));
				}
			}
			else {
				$user_id = $user->ID;
				global $blog_id;
				if ( is_multisite() && $blog_id && !is_user_member_of_blog($user_id) ) {
					// Add user to the current blog (in multisite)
					add_user_to_blog($blog_id, $user_id, $role);
					$this->users_count++;
				}
			}
			return $user_id;
		}

		/**
		 * Process the state code before saving it
		 * 
		 * @since 3.27.1
		 * 
		 * @param string $state_code State code
		 * @param string $country Country code
		 * @return string State code
		 */
		private function process_state_code($state_code, $country) {
			if ( $country != 'US') {
				$state_code = preg_replace('/^' . preg_quote($country) . '-/', '', $state_code); // Remove the country prefix from the state code
			}
			return $state_code;
		}
		
		/**
		 * Get the PrestaShop employees
		 * 
		 * @return array of employees
		 */
		private function get_employees() {
			$employees = array();

			$prefix = $this->plugin->plugin_options['prefix'];
			$sql = "
				SELECT id_employee, lastname, firstname, email, passwd
				FROM ${prefix}employee
				WHERE active = 1
			";
			$employees = $this->plugin->prestashop_query($sql);

			return $employees;
		}

		/**
		 * Get the PrestaShop customers
		 * 
		 * @param int $limit Number of customers max
		 * @return array of customers
		 */
		private function get_customers($limit=1000) {
			$customers = array();

			$last_customer_id = (int)get_option('fgp2wc_last_customer_id'); // to restore the import where it left
			$prefix = $this->plugin->plugin_options['prefix'];
			if ( $this->plugin->column_exists('customer', 'website') ) {
				$website_field = 'c.website';
			} else {
				$website_field = "'' AS website";
			}
			$deleted = '';
			if ( $this->plugin->column_exists('customer', 'deleted') ) {
				$deleted = 'AND c.deleted = 0';
			}
			$sql = "
				SELECT c.id_customer, c.firstname, c.lastname, c.email, c.passwd, $website_field, c.date_add
				FROM ${prefix}customer c
				WHERE c.active = 1
				AND c.id_customer > '$last_customer_id'
				$deleted
				ORDER BY c.id_customer
				LIMIT $limit
			";
			$sql = apply_filters('fgp2wc_get_customers_sql', $sql);

			$customers = $this->plugin->prestashop_query($sql);

			return $customers;
		}

		/**
		 * Get a PrestaShop customer address
		 * 
		 * @param int $customer_id Customer ID
		 * @return array Address fields
		 */
		private function get_customer_address($customer_id) {
			$address = array();

			$prefix = $this->plugin->plugin_options['prefix'];
			$sql = "
				SELECT a.company, a.lastname, a.firstname, a.address1, a.address2, a.postcode, a.city, a.phone, a.phone_mobile,
				c.iso_code AS country, s.iso_code AS state
				FROM ${prefix}address a
				LEFT JOIN ${prefix}country c ON c.id_country = a.id_country
				LEFT JOIN ${prefix}state s ON s.id_state = a.id_state
				WHERE a.id_customer = '$customer_id'
				AND a.active = 1
				AND a.deleted = 0
				LIMIT 1
			";
			$result = $this->plugin->prestashop_query($sql);
			if ( count($result) > 0 ) {
				$address = $result[0];
			}

			return $address;
		}

	}
}
